"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
class Persistence {
    constructor(name, fs, statePath) {
        this.name = name;
        this.fs = fs;
        this.statePath = statePath;
        this.lastKnownFileContent = '';
        this.saveState = () => {
            const serializedState = JSON.stringify(this.model.toJSON(), null, 4);
            if (serializedState !== this.lastKnownFileContent) {
                this.lastKnownFileContent = serializedState;
                this.fs.saveFile(this.statePath, serializedState);
                // .then(() => console.log(` ${this.name} state file saved at ${this.fs.baseUrl}/${this.statePath}`));
            }
        };
    }
    init(model) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.model = model;
            const changeListener = (e) => {
                if (e.fullPath === this.statePath && this.lastKnownFileContent !== e.newContent) {
                    this.lastKnownFileContent = e.newContent;
                    console.log(`Detected change in ${this.name} state file. loading state...`);
                    this.model.fromJSON(JSON.parse(e.newContent));
                }
            };
            const intervalHandle = setInterval(this.saveState, 1000);
            this.disposer = () => {
                clearInterval(intervalHandle);
                this.fs.events.off('fileChanged', changeListener);
            };
            this.fs.events.on('fileChanged', changeListener);
            try {
                const content = yield this.fs.loadTextFile(this.statePath);
                console.log(`loading existing ${this.name} state from ${this.fs.baseUrl}/${this.statePath}`);
                this.lastKnownFileContent = content;
                this.model.fromJSON(JSON.parse(content));
            }
            catch (_) {
                console.log(`no existing ${this.name} state file found at ${this.fs.baseUrl}/${this.statePath}`);
            }
        });
    }
}
exports.Persistence = Persistence;
//# sourceMappingURL=persistency.js.map