"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_utils_1 = require("../session/command-utils");
const empty_epsilon_js_1 = require("empty-epsilon-js");
function init(builder, repair, entries) {
    builder
        .descriptor({
        title: "E.C.R commands"
    })
        .command('stop', {
        title: 'stop',
        description: `Deactivate the automatic repair subsystem procedure. Re-allocate computation resources to ship cores`,
        returns: 'ui.message',
        syntax: 'stop repair'
    })
        .handler(command_utils_1.withUser(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield repair.stopRepair();
        return `Automatic repair routine inactive. Computation resources available to ship cores`;
    }))).parent()
        .command('do', {
        title: 'do',
        description: `Send a custom order to the ECR auto-mechanic subsystem`,
        returns: 'ui.message',
        syntax: 'do (command string)'
    })
        .handler(command_utils_1.withUser((request, user) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const queryData = command_utils_1.normInputString(request.data("command"));
        const result = yield entries.query(user, queryData);
        if (result) {
            return `${result} ${Date.now() % 2 ? 'beer' : 'coffee'}?`;
        }
        else {
            throw new Error('ECR clearance insufficient');
        }
    }))).parent();
    Array.from(Array(empty_epsilon_js_1.ESystem.COUNT))
        .forEach((_, systemId) => builder
        .command(`repair_${empty_epsilon_js_1.ESystem[systemId]}`, {
        title: "repair",
        description: `Activate and assign the automatic repair subsystem to ${empty_epsilon_js_1.ESystem[systemId]}. Requires computation resources`,
        returns: "ui.message",
        syntax: `repair ${empty_epsilon_js_1.ESystem[systemId]}`
    })
        .handler(command_utils_1.withUser(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield repair.beginRepair(systemId);
        return `System ${empty_epsilon_js_1.ESystem[systemId]} automatic repair routine engaged`;
    }))).parent());
}
exports.default = init;
//# sourceMappingURL=commands.js.map