"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const empty_epsilon_js_1 = require("empty-epsilon-js");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
exports.repair_per_second = 0.007;
exports.heat_per_second = 0.05 * /* heat_sanity_factor */ 0.28;
exports.damage_per_second_on_overheat = 0.08;
exports.min_reactor_health = -0.89;
const powerQueries = empty_epsilon_js_1.ESystemNames.map((system) => `getPlayerShip(-1):getSystemPower('${system}')`);
class EcrDriver {
    constructor(eeDriver, pulse) {
        this.eeDriver = eeDriver;
        this.pulse = pulse;
        this.queryPowerOfAllSystems = () => {
            return rxjs_1.of(...powerQueries)
                .pipe(operators_1.map((query, system) => tslib_1.__awaiter(this, void 0, void 0, function* () { return ({ system, power: yield this.eeDriver.query(query) }); })))
                .pipe(operators_1.mergeMap(msg => rxjs_1.from(msg)));
        };
        this.powerUpdates = this.pulse.pipe(operators_1.mergeMap(this.queryPowerOfAllSystems));
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield Promise.all([
                // kill the fake repair crew
                this.eeDriver.command(`getPlayerShip(-1):setRepairCrewCount({0})`, ['0']),
                // eeDriver.command(`getPlayerShip(-1):setAutoCoolant({0})`, ['false']);
                this.eeDriver.addSystemFeature('RepairRate', `
if value > 0 then
    ship:setSystemHealth(system, math.min(1, ship:getSystemHealth(system) + (value * ${exports.repair_per_second} * delta)))
end
`),
                this.eeDriver.addSystemFeature('HeatRate', `
if value > 0 then
    local newHeat = ship:getSystemHeat(system) + (value * ${exports.heat_per_second} * delta)
    if newHeat > 1 then
        ${
                /*
                // Heat damage is specified as damage per second while overheating.
                // Calculate the amount of overheat back to a time, and use that to
                // calculate the actual damage taken.
                 */ ''}
        local newHealth = ship:getSystemHealth(system) - (newHeat -1) * ${exports.damage_per_second_on_overheat / exports.heat_per_second}
        if system == '${empty_epsilon_js_1.ESystem[empty_epsilon_js_1.ESystem.Reactor]}' and newHealth < ${exports.min_reactor_health} then 
            newHealth = ${exports.min_reactor_health}
        end
        ship:setSystemHealth(system, math.max(-1, newHealth))
        newHeat = 1 
    end
    ship:setSystemHeat(system, newHeat)
end
`),
                this.eeDriver.addSystemFeature('MaxPower', `
local power = ship:getSystemPower(system)
if power > value then
    ship:setSystemPower(system, value)
end
`),
                this.eeDriver.exec(`
local ship = getPlayerShip(-1)
if not ship.maxReactorHealth then
    ship.maxReactorHealth = true
    
    function setMaxReactorHealth(delta)
        local health = ship:getSystemHealth('${empty_epsilon_js_1.ESystem[empty_epsilon_js_1.ESystem.Reactor]}')
        if health < ${exports.min_reactor_health} then
            ship:setSystemHealth('${empty_epsilon_js_1.ESystem[empty_epsilon_js_1.ESystem.Reactor]}', ${exports.min_reactor_health})
        end
    end

    table.insert(ship._daedalus_hooks, setMaxReactorHealth) 
end
`)
            ]);
        });
    }
    setRepairRate(system, repairRate) {
        return this.eeDriver.command(`getPlayerShip(-1):setSystemRepairRate('${empty_epsilon_js_1.ESystem[system]}', {0})`, [repairRate.toFixed(4)]);
    }
    setHeatRate(system, heatRate) {
        return this.eeDriver.command(`getPlayerShip(-1):setSystemHeatRate('${empty_epsilon_js_1.ESystem[system]}', {0})`, [heatRate.toFixed(4)]);
    }
    setMaxPower(system, maxPower) {
        return this.eeDriver.command(`getPlayerShip(-1):setSystemMaxPower('${empty_epsilon_js_1.ESystem[system]}', {0})`, [maxPower.toFixed(4)]);
    }
}
exports.EcrDriver = EcrDriver;
//# sourceMappingURL=driver.js.map