"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const rpc = require("json-rpc2");
function expose(module, options) {
    const server = rpc.Server.$create();
    server.expose('beginRepair', function ([id], _, callback) {
        try {
            module.beginRepair(id);
            callback();
        }
        catch (e) {
            callback(e);
        }
    });
    server.expose('stopRepair', function (_args, _, callback) {
        try {
            module.stopRepair();
            callback();
        }
        catch (e) {
            callback(e);
        }
    });
    const port = options.port || 56667;
    console.log(`providing RPC of ECR in port ${port} and host ${options.hostname}`);
    return server.listen(port, options.hostname);
}
exports.expose = expose;
class EcrModuleClient {
    constructor(options) {
        this.options = options;
        const port = options.port || 56667;
        this.client = rpc.Client.$create(port, options.hostname);
        console.log(`setting up RPC of ECR in port ${this.options.port || 56667} and host ${this.options.hostname}`);
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            console.warn('using TCP for RPC! TCP does not reconnect!');
            this.con = yield new Promise((resolve, reject) => this.client.connectSocket((err, con) => {
                console.log('RPC2');
                if (err) {
                    console.log(`ERROR consuming RPC of ECR in port ${this.options.port || 56667} and host ${this.options.hostname}`, err);
                    reject(err);
                }
                else {
                    resolve(con);
                }
            }));
            console.log(`consuming RPC of ECR in port ${this.options.port || 56667} and host ${this.options.hostname}`);
        });
    }
    beginRepair(id) {
        if (this.con) {
            return new Promise((resolve, reject) => this.con.call('beginRepair', [id], function (err, result) {
                if (err) {
                    reject(new Error(err));
                }
                else {
                    resolve(result);
                }
            }));
        }
        else {
            return new Promise((resolve, reject) => this.client.call('beginRepair', [id], function (err, result) {
                if (err) {
                    reject(new Error(err.message));
                }
                else {
                    resolve(result);
                }
            }));
            // throw new Error("EcrModuleClient not initialized but calling beginRepair()")
        }
    }
    stopRepair() {
        if (this.con) {
            return new Promise((resolve, reject) => this.con.call('stopRepair', [], function (err, result) {
                err ? reject(new Error(err)) : resolve(result);
            }));
        }
        else {
            return new Promise((resolve, reject) => this.client.call('stopRepair', [], function (err, result) {
                if (err) {
                    reject(new Error(err.message));
                }
                else {
                    resolve(result);
                }
            }));
            //throw new Error("EcrModuleClient not initialized but calling beginRepair()")
        }
    }
    end() {
        this.con && this.con.end();
    }
}
exports.EcrModuleClient = EcrModuleClient;
//# sourceMappingURL=rpc.js.map