"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_utils_1 = require("../session/command-utils");
function init(builder, entries) {
    builder
        .descriptor({
        title: "excalibur"
    })
        .command("open", {
        title: "open",
        description: `Open an information entry for display only`,
        returns: "ui.markdown",
        syntax: "open (entryName string)"
    })
        .handler(command_utils_1.withUser((request, user) => {
        const entryName = request.data("entryName");
        const result = entries.open(user, entryName);
        if (result) {
            return result;
        }
        else {
            throw new Error('File not found');
        }
    }))
        .parent()
        .command('list', {
        title: 'list',
        description: `List all available information entries`,
        returns: 'list<string>',
        syntax: 'list'
    })
        .handler(command_utils_1.withUser(() => {
        return entries.list();
    })).parent()
        .command('query', {
        title: 'query',
        description: `Send a custom query to the Excalibur deep analysis subsystem`,
        returns: 'ui.markdown',
        syntax: 'query (queryData string)'
    })
        .handler(command_utils_1.withUser((request, user) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        const queryData = command_utils_1.normInputString(request.data("queryData"));
        const result = yield entries.query(user, queryData);
        if (result) {
            return result;
        }
        else {
            throw new Error('Excalibur clearance insufficient');
        }
    })));
}
exports.default = init;
//# sourceMappingURL=commands.js.map