"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fugazi = require("@fugazi/connector");
const kissfs_1 = require("kissfs");
const commands_1 = require("./excalibur/commands");
const commands_2 = require("./ecr/commands");
const path_1 = require("path");
const entries_1 = require("./excalibur/entries");
const rpc_1 = require("./ecr/rpc");
const resolve = require("resolve");
function hookIntoFugaziConnector() {
    const sessionModule = require('@fugazi/connector/scripts/bin/middleware/session');
    const oldMW = sessionModule.middleware;
    sessionModule.middleware = (app, config) => {
        // set 1 hour timeout
        app.use((ctx, next) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            ctx.req.setTimeout(60 * 60 * 1000);
            yield next();
        }));
        return oldMW(app, config);
    };
}
class FugaziServices {
    constructor(fs, options) {
        this.fs = fs;
        // application BL modules
        //    this.users = new Users(fs);
        this.entries = new entries_1.Entries(fs);
        this.ecrCommands = new entries_1.Entries(fs, {
            entriesPath: 'FAKE_PATH_IGNORE_WARNING_ITS_OK_IM_AN_ENGINEER',
            queriesPath: 'commands',
            queriesPrefix: 'command',
            generateQueryBody() {
                return `DONE!`;
            }
        });
        this.ecr = new rpc_1.EcrModuleClient({ hostname: options.ecrHost, port: options.ecrPort });
        // connector API
        const builder = new fugazi.ConnectorBuilder();
        builder.server()
            .port(options.port)
            .host(options.hostname)
            .folder(path_1.parse(resolve.sync('daedalus-fugazi-webclient/index.html')).dir)
            .session({ keygrip: ['abracadabra!'] });
        commands_1.default(builder.module("excalibur"), this.entries);
        commands_2.default(builder.module("ecr"), this.ecr, this.ecrCommands);
        //   initLogin(builder.module("session"), this.users);
        this.connector = builder.build();
        this.connector._server.koa.use((ctx, next) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            ctx.req.setTimeout(0);
            yield next();
        }));
    }
    init() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.entries.init();
            yield this.ecrCommands.init();
            yield this.connector.start();
            // DONT INIT ECR RPC, KEEP IT IN HTTP MODE BECAUSE TCP DOES NOT RECONNECT
            // await this.ecr.init();
            // connector.logger.info("started");
        });
    }
    close() {
        this.entries.destroy();
        this.ecrCommands.destroy();
        this.ecr.end();
        this.connector.stop();
    }
}
exports.FugaziServices = FugaziServices;
process.on('uncaughtException', function (err) {
    console.error(err.message);
    console.error(err.stack);
});
function main(optionsArg) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const fs = yield (new kissfs_1.LocalFileSystem(optionsArg.resources)).init();
        hookIntoFugaziConnector();
        const fugaziServices = new FugaziServices(fs, optionsArg);
        const exit = fugaziServices.close.bind(fugaziServices);
        process.once('exit', exit);
        process.once('SIGINT', exit.bind(null, true, 2));
        process.once('SIGTERM', exit.bind(null, true, 15));
        yield fugaziServices.init();
    });
}
exports.main = main;
//# sourceMappingURL=fugazi.js.map