"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const logic_1 = require("../../src/ecr/logic");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
function getLinearDeriviation(sample, { iterations, graceFactor, tickInterval }) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const sampleInterval = tickInterval * 2 / graceFactor;
        const measurements = [];
        let startValue = yield sample();
        const start = Date.now();
        for (let i = sampleInterval; i < sampleInterval * iterations; i += sampleInterval) {
            setTimeout(() => tslib_1.__awaiter(this, void 0, void 0, function* () {
                measurements.push(((yield sample()) - startValue) / (Date.now() - start));
            }), i);
        }
        yield new Promise(res => setTimeout(res, sampleInterval * (iterations + 1)));
        // assert the deriviation is linear and average it
        return measurements.reduce((avg, curr, i) => {
            chai_1.expect(curr).to.be.approximately(avg, avg * graceFactor);
            return (i * avg + curr) / (i + 1);
        });
    });
}
exports.getLinearDeriviation = getLinearDeriviation;
function getLinearOverloadDeriviation(status, graceFactor) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        return yield getLinearDeriviation(() => tslib_1.__awaiter(this, void 0, void 0, function* () { return status.overload; }), { iterations: 3, graceFactor, tickInterval: logic_1.EcrLogic.tickInterval });
    });
}
exports.getLinearOverloadDeriviation = getLinearOverloadDeriviation;
/**
 * match floating point number up to 0.1% deviation
 */
function approx(val) {
    return sinon_1.match((v) => (v < val * 1.001) && (v > val * 0.999), `approx. ${val.toFixed(2)}`);
}
exports.approx = approx;
//# sourceMappingURL=test-kit.js.map